<?php
/*
http://www.payline.com

Payline module for Ubercart 3.x under Drupal 7.x - v2.0 - August 2012

Copyright (c) 2012 Monext
*/

require_once(drupal_get_path('module', 'uc_payline') .'/configuration/options.php');
require_once(drupal_get_path('module', 'uc_payline') .'/lib/paylineSDK.php');

/**
 * Customer return after payline validation
 */
function uc_payline_response($notif = FALSE) {

	global $user;
	$payline = new paylineSDK(
		variable_get('uc_payline_merchant_id', ''),
		variable_get('uc_payline_access_key', ''),
		variable_get('uc_payline_proxy_host', NULL),
		variable_get('uc_payline_proxy_port', NULL),
		variable_get('uc_payline_proxy_login', ''),
		variable_get('uc_payline_proxy_password', ''),
		variable_get('uc_payline_environment', '0')
	);

	// recuperation du token de la commande
	if (isset($_POST['token'])) {
		$token = $_POST['token'];
	}
	elseif (isset($_GET['token'])) {
		$token = $_GET['token'];
	}
	else {
		drupal_set_message(t('You are not allowed to see this page, please, return to homepage.'));
		watchdog('uc_payline', 'Error: access response page with no token, ip: @ip', array('@ip' => ip_address()), WATCHDOG_WARNING);
		drupal_goto('cart');
	}

	// response depuis payline
	$array['version'] = '2';
	$array['token'] = $token;
	$response = $payline->getWebPaymentDetails($array);

	// debug
	if (variable_get('uc_payline_debug', FALSE) == TRUE) {
		drupal_set_message('<pre>Response: '. print_r($response, TRUE) .'</pre>');
	}

	// traitement de la reponse
	if(isset($response['result']['code'])){
		$code = $response['result']['code'];
		$order_id = $response['privateDataList']['privateData']['value'];
		$order = uc_order_load($order_id);

		if ($order) {
			$result = db_select('uc_payline', 'ptk')
			->fields('ptk')
			->condition('order_id', $order_id,'=')
			->condition('result','NEW','=')
			->execute()
			->fetchAssoc();

			if (strcmp($result['token'], $token) == 0) {
				if ($notif) {
					$payline->writeTrace("order ".$order_id." - payment notification : ".$response['result']['longMessage']." (code ".$response['result']['code'].")");
					return uc_payline_auto($order, $response);
				}elseif ($code == '02319') {
					$payline->writeTrace("order ".$order_id." - Payment cancelled by the buyer");
					db_update('uc_payline')
						->fields(array('result' => 'CANCEL'))
						->condition('token',$token,'=')
						->execute();
					uc_payline_cancel($order);
				}elseif ($code == '00000') {
					$payline->writeTrace("order ".$order_id." - payment success : transaction ".$response['transaction']['id']);
					db_update('uc_payline')
						->fields(array('result' => $response['transaction']['id']))
						->condition('token',$token,'=')
						->execute();
					return uc_payline_complete($order, $response);
				}else {
					$errors = array('!id' => $order->order_id, '!code' => $response['result']['code'], '@msg' => $response['result']['longMessage']);
					uc_order_comment_save($order->order_id, 0, t('Error [!code] order !id: @msg.', $errors));
					watchdog('uc_payline', "Error [!code] order !id: @msg.", $errors, WATCHDOG_WARNING, NULL);
					drupal_set_message(t('Your payment was refused (@msg), please try again.'), $errors);
					$payline->writeTrace("order ".$order_id." - payment refused : ".$response['result']['longMessage']." (code ".$response['result']['code'].")");
					db_update('uc_payline')
						->fields(array('result' => 'REFUSED'))
						->condition('token',$token,'=')
						->execute();
					drupal_goto('cart');
				}
			}else {
				drupal_set_message(t('An error has occured after payment, please contact us.'), 'error');
				watchdog('uc_payline', "Token error on order !id: !token1 / !token2.", array('!id' => $order_id, '!token1' => $token, '!token2' => $result['token']), WATCHDOG_WARNING, NULL);
				$payline->writeTrace("ERROR - token ".$token." is unkwown for order ".$order_id." (expected : ".$result['token'].")");
				db_update('uc_payline')
					->fields(array('result' => 'ERROR'))
					->condition('token',$token,'=')
					->execute();
				drupal_goto('cart');
			}
		}else {
			drupal_set_message(t('An error has occured after payment, please contact us.'), 'error');
			watchdog('uc_payline', "No order_id found. token: !token", array('!token' => $token), WATCHDOG_WARNING, NULL);
			$payline->writeTrace("ERROR - unable to load order ".$order_id." (token $token)");
			db_update('uc_payline')
				->fields(array('result' => 'ERROR'))
				->condition('token',$token,'=')
				->execute();
			drupal_goto('cart');
		}
	}else{
		drupal_set_message(t('An error has occured after payment, please contact us.'), 'error');
		watchdog('uc_payline', "Unable to call getWebPaymentDetails for token !token.", array('!token' => $token), WATCHDOG_WARNING, NULL);
		$payline->writeTrace("ERROR - unable to call getWebPaymentDetails for token ".$token." : ".$response);
		db_update('uc_payline')
			->fields(array('result' => 'ERROR'))
			->condition('token',$token,'=')
			->execute();
		drupal_goto('cart');

	}
}

/**
 * Handle payline payment validation
 */
function uc_payline_complete($order, $response) {

	// Check status
	if ($order->order_status != 'in_checkout' && $order->order_status != 'pending') {
		$errors[] = t('Order not in checkout or pending: ') . $order->order_status;
	}
	if ($order->order_status == 'payment_received') {
		$errors[] = t('Try to revalide a payment.');
	}
	if (!empty($errors)) {
		// Log errors to order and watchdog.
		$error_data = array('!id' => $order->order_id, '@error' => implode(" - ", $errors));
		uc_order_comment_save($order->order_id, 0, t('Order !id caused error: @error', $error_data), 'admin');
		watchdog('uc_payline', 'Order !id caused error: @error', $error_data, WATCHDOG_ERROR);
		// debug
		if (variable_get('uc_payline_debug', FALSE) == TRUE) {
			$output = t('Order !id caused error: @error', $error_data);
			print $output;
		}
		drupal_goto('cart');
	}

	// tout est ok, validation de la commande
	$variables = array(
                '!tid' => $response['transaction']['id'],
                '!tdate' => $response['transaction']['date'],
                '!auth_id' => $response['authorization']['number'],
	);
	$comment = t('Paid by payline transaction: !tid - !tdate authorization: !auth_id.', $variables);
	// validation du paiement
	uc_payment_enter($order->order_id, 'uc_payline', $response['payment']['amount'] / 100, 0, NULL, $comment);
	// commentaires
	uc_order_comment_save($order->order_id, 0, t('Order created through website. Payline transaction id: !tid', $variables));
	uc_order_comment_save($order->order_id, 0, t('Payment accepted. Payline transaction id: !tid', $variables), 'order');
	// validation de la commande et redirection
	$output = uc_cart_complete_sale($order, variable_get('uc_new_customer_login', FALSE));
	// check if alternate checkout completion page is set
	$page = variable_get('uc_cart_checkout_complete_page', '');
	if (!empty($page)) {
		drupal_goto($page);
	}
	return $output;
}


/**
 * Handle payline cancel payment
 */
function uc_payline_cancel($order) {

	global $user;
	unset($_SESSION['cart_order']);
	uc_order_comment_save($order->order_id, $user->uid, t('This order has been explicitly canceled by the user.'), 'order');

	if (uc_order_update_status($order->order_id, uc_order_state_default('canceled'))) {
		drupal_set_message(t('Your order has been canceled.'), 'status');
	}
	else {
		drupal_set_message(t('An error has occured during payment, please contact us.'), 'error');
		watchdog('uc_payline', "The order @order couldn't be marked as 'Canceled'.", array('@order' => $order->order_id), WATCHDOG_WARNING, NULL);
	}
	drupal_goto('cart');
}

/**
 * auto notification from payline
 */
function uc_payline_auto($order, $response) {
	$montant = $response['payment']['amount'];

	// check order
	if (!$order) {
		$errors[] = t('Invalid order !id.', array('!id' => $order->order_id));
	}
	else {
		// Check status
		if ($order->order_status != 'in_checkout' && $order->order_status != 'pending') {
			$errors[] = t('Order not in checkout or pending: ') . $order->order_status;
		}
		if ($order->order_status == 'payment_received') {
			$errors[] = t('Try to revalide a payment.');
		}
		// Check price
		if ($order->order_total*100 != $montant) {
			$errors[] = t('Incorrect price: @price1 / @price2.', array('@price1' => $order->order_total*100, '@price2' => $montant));
		}
		// Check error code
		if ($erreur != "00000") {
			$errors[] = t('Error [!code] order !id: @msg.', array('!id' => $order->order_id, '!code' => $response['result']['code'], '@msg' => $response['result']['longMessage']));
		}
	}
	if (!empty($errors)) {
		// Log errors to order and watchdog.
		$error_data = array('!id' => $order->order_id, '@error' => implode(" - ", $errors));
		// admin comment
		uc_order_comment_save($order->order_id, 0, t('Autoresponse error on order !id: @error', $error_data));
		watchdog('uc_payline', 'Order !id autoresponse caused error: @error', $error_data, WATCHDOG_ERROR);

		// debug
		if (variable_get('uc_payline_debug', FALSE) == TRUE) {
			$output = t('Order !id caused error: @error', $error_data);
			print $output;
		}
		exit();
	}

	// if everything ok, add comment (later: use auto to handle recurring payment)
	$comment = t('Payline notify transaction: @tid - @tdate authorization: @auth_id.',
	array(
                '!tid' => $response['transaction']['id'],
                '!tdate' => $response['transaction']['date'],
                '!auth_id' => $response['authorization']['number'],
	));
	// admin comment
	uc_order_comment_save($order->order_id, 0, $comment);
	// debug
	if (variable_get('uc_payline_debug', FALSE) == TRUE) {
		print $comment;
	}
	exit();
}

/**
 * Get payment detail form
 */
function uc_payline_get_details($order_id) {
	if ($order_id) {
		drupal_set_title(t('Get payment detail on order !oid', array('!oid' => $order_id)));
		$token = db_result(db_query("SELECT token FROM {uc_payline} WHERE order_id = %d", $order_id));
		$output = t('Token:  !token', array('!token' => $token));
		$payline = new paylineSDK(
		variable_get('uc_payline_merchant_id', ''),
		variable_get('uc_payline_access_key', ''),
		variable_get('uc_payline_proxy_host', NULL),
		variable_get('uc_payline_proxy_port', NULL),
		variable_get('uc_payline_proxy_login', ''),
		variable_get('uc_payline_proxy_password', ''),
		variable_get('uc_payline_environment', '0')
		);
		$array['version'] = '2';
		$array['token'] = $token;
		$response = $payline->getWebPaymentDetails($array);
		if (isset($response)) {
			foreach ($response as $data => $resp) {
				$output .= '<h2>'. $data .'</h2>';
				foreach ($resp as $k => $v) {
					is_array($v) ? $output .= '<h3>'. $k .'</h3>'. implode(' :', $v) .'<br/>' : $output .= $k .' : '. $v .'<br/>';
				}
			}
		}
		else {
			drupal_set_message(t('Sorry, no answer found with this token: !token', array('!token' => $token)), 'error');
		}
		$output .= l('retour', 'admin/store/orders/view');
		return $output;
	}
	else {
		drupal_goto('admin/store/orders/view');
	}
}
